"use strict";(self.webpackChunk_JUPYTERLAB_CORE_OUTPUT=self.webpackChunk_JUPYTERLAB_CORE_OUTPUT||[]).push([[6976,3349],{66976:(e,t,a)=>{a.r(t),a.d(t,{default:()=>r});var i=a(59110),n=a(92233);class o{constructor(e){this._initPromise=new n.PromiseDelegate,this._initialized=!1,this._url=e.url,this._config=e.config}typeset(e){this._initialized||this._init(),this._initPromise.promise.then((()=>{MathJax.Hub.Queue(["Typeset",MathJax.Hub,e]);try{MathJax.Hub.Queue(["Require",MathJax.Ajax,"[MathJax]/extensions/TeX/AMSmath.js"],(()=>{MathJax.InputJax.TeX.resetEquationNumbers()}))}catch(e){console.error("Error queueing resetEquationNumbers:",e)}}))}_init(){const e=document.getElementsByTagName("head")[0],t=document.createElement("script");t.type="text/javascript",t.src=`${this._url}?config=${this._config}&amp;delayStartupUntil=configured`,t.charset="utf-8",e.appendChild(t),t.addEventListener("load",(()=>{this._onLoad()})),this._initialized=!0}_onLoad(){MathJax.Hub.Config({tex2jax:{inlineMath:[["$","$"],["\\(","\\)"]],displayMath:[["$$","$$"],["\\[","\\]"]],processEscapes:!0,processEnvironments:!0},displayAlign:"center",CommonHTML:{linebreaks:{automatic:!0}},"HTML-CSS":{availableFonts:[],imageFont:null,preferredFont:null,webFont:"STIX-Web",styles:{".MathJax_Display":{margin:0}},linebreaks:{automatic:!0}},skipStartupTypeset:!0,messageStyle:"none"}),MathJax.Hub.Register.StartupHook("End Config",(()=>{var e,t,a,i,n,o;null===(a=null===(t=null===(e=MathJax.Hub)||void 0===e?void 0:e.config)||void 0===t?void 0:t.MathEvents)||void 0===a||delete a.styles[".MathJax_Hover_Arrow:hover span"],null===(o=null===(n=null===(i=MathJax.Hub)||void 0===i?void 0:i.config)||void 0===n?void 0:n.MathMenu)||void 0===o||delete o.styles[".MathJax_MenuClose:hover span"]})),MathJax.Hub.Configured(),this._initPromise.resolve(void 0)}}const s={id:"@jupyterlab/mathjax2-extension:plugin",autoStart:!0,provides:a(12299).ILatexTypesetter,activate:()=>{const[e,t]=["fullMathjaxUrl","mathjaxConfig"],a=i.PageConfig.getOption(e),n=i.PageConfig.getOption(t);if(!a){const a=`${s.id} uses '${e}' and '${t}' in PageConfig to operate but '${e}' was not found.`;throw new Error(a)}return new o({url:a,config:n})}},r=s}}]);